#ifndef __QTCLINTERP_H__
#define __QTCLINTERP_H__
#include <qobject.h>

#include "QTclFactory.h"

struct Tcl_Interp;
typedef int (Tcl_CmdProc) (void * clientData, Tcl_Interp *interp, int argc, char *argv[]);

class QTclInterp : public QObject {
  Q_OBJECT
  Q_PROPERTY( bool debug READ debug WRITE setDebug RESET unsetDebug )
  Q_PROPERTY( int interactive READ interactive WRITE setInteractive RESET unsetInteractive)
  Q_PROPERTY( QCString result READ result DESIGNABLE false STORED false )
public:
  QTclInterp(QObject *parent,const char *name="tcl");
  ~QTclInterp();
  int interactive() const { return _interactive; }
  bool debug() const { return _debug; }
  QCString result() const;
  Tcl_Interp *interp() const { return _interp; }
public slots:
  void setDebug();
  void setDebug(bool debug);
  void unsetDebug();
  void setInteractive() { setInteractive(2); }
  void setInteractive(int interactive);
  void unsetInteractive();
  int  eval(const char *cmd);
  int  evalFile(const char *fileName);
private:
  Tcl_Interp *_interp;
  bool _debug;
  int _interactive;
public:
  static const char*registerObject(Tcl_Interp *interp,QObject *o,char *buffer,int size,int deep=0);
  static const char*objectName(const QObject *w,char *buffer,int size);
  static int getObjectByName(Tcl_Interp *interp,const char *name,QObject **result,const char *className=0);
  static int getWidgetByName(Tcl_Interp *interp,const char *name,QWidget **result);
  static int prepareForCreate(Tcl_Interp *interp,const char *name,const char *className,QObject **parent,char *buffer,int size);
  static int appendChilds(Tcl_Interp *interp,const QObject *w,int deepMode,const char *className);
  static int appendGlobalMethodNames(Tcl_Interp *interp);
  static int appendClassNames(Tcl_Interp *interp);
  static int appendClassMethodNames(Tcl_Interp *interp);
  static int appendName(Tcl_Interp *interp,const QObject *w);
  static int registerMethods(const char *className,Tcl_CmdProc *proc);
  static int registerClassMethods(const char *className,Tcl_CmdProc *proc);
  static int registerGlobalMethod(const char *factoryName,Tcl_CmdProc *proc);
  static int registerFactory(QTclFactory *factory);
  static Tcl_CmdProc *findMethods(const QObject *o);
  static Tcl_CmdProc *findClassMethods(const char *className);
  static Tcl_CmdProc *findClassMethods(const QObject *o);
  static Tcl_CmdProc *findGlobalMethod(const char *factoryName);
  /* GlobalFactory */
  static QObject *create(const char *className,QObject *parent,const char *name);
  /* Evaluate */
  static void eval(Tcl_Interp *interp,const char *command,int argc,...);
};

#endif
